@@ -1002,3 +1002,46 @@ class HdDB:
             return result.to_dict(orient="records")[0]["config"]
         except duckdb.Error as e:
             raise QueryError(f"Error executing query: {e}")
+    
+    @attach_motherduck
+    def create_geo_table(self, org: str, db: str, tbl: str, df: pd.DataFrame):
+        try:
+            self.execute("BEGIN TRANSACTION;")
+
+            self.execute("INSTALL spatial;")
+            self.execute("LOAD spatial;")
+
+            self.conn.register("df", df)
+
+            # Crear la tabla en el esquema actual
+            create_query = f'CREATE TABLE "{org}__{db}"."{tbl}" (id VARCHAR, properties JSON, geometry GEOMETRY);'
+            
+            insert_query = f'INSERT INTO "{org}__{db}"."{tbl}" SELECT id, properties, ST_GeomFromWKB(geometry) FROM df;'
+
+            self.execute(create_query)
+            self.execute(insert_query)
+
+            self.execute("COMMIT;")
+
+            return True
+        except duckdb.Error as e:
+            self.execute("ROLLBACK;")
+            raise QueryError(f"Error executing query: {e}")
+        
+    @attach_motherduck
+    def get_geo_table_by_shape(self, org: str, db: str, tbl: str, shape: tuple):
+        try:
+            self.execute("BEGIN TRANSACTION;")
+
+            self.execute("INSTALL spatial;")
+            self.execute("LOAD spatial;")
+
+            query = f'SELECT id, properties, ST_AsWKB(geometry) as geometry FROM "{org}__{db}"."{tbl}" WHERE ST_Within(geometry, ST_MakeEnvelope({shape[0]}, {shape[1]}, {shape[2]}, {shape[3]}))'
+            result = self.execute(query).fetchdf()
+
+            self.execute("COMMIT;")
+
+            return result
+        except duckdb.Error as e:
+            self.execute("ROLLBACK;")
+            raise QueryError(f"Error executing query: {e}")
